@@ -11,7 +11,20 @@ sym_find(VALUE dummy, VALUE sym)
 static VALUE
 sym_pinneddown_p(VALUE dummy, VALUE sym)
 {
-    return rb_check_id(&sym) ? Qtrue : Qfalse;
+    ID id = rb_check_id(&sym);
+    if (!id) return Qnil;
+#ifdef ULL2NUM
+    return ULL2NUM(id);
+#else
+    return ULONG2NUM(id);
+#endif
+}
+
+static VALUE
+sym_iv_get(VALUE dummy, VALUE obj, VALUE name)
+{
+    const char *n = StringValueCStr(name);
+    return rb_iv_get(obj, n);
 }
 
 void
@@ -21,5 +34,6 @@ Init_symbol(void)
     VALUE klass = rb_define_class_under(mBug, "Symbol", rb_cSymbol);
     rb_define_singleton_method(klass, "find", sym_find, 1);
     rb_define_singleton_method(klass, "pinneddown?", sym_pinneddown_p, 1);
+    rb_define_singleton_method(klass, "iv_get", sym_iv_get, 2);
     TEST_INIT_FUNCS(init);
 }
