@@ -100,6 +100,40 @@ describe "C-API Exception function" do
     end
   end
 
+  describe "rb_syserr_new" do
+    it "returns system error with default message when passed message is NULL" do
+      exception = @s.rb_syserr_new(Errno::ENOENT::Errno, nil)
+      exception.class.should == Errno::ENOENT
+      exception.message.should include("No such file or directory")
+      exception.should.is_a?(SystemCallError)
+    end
+
+    it "returns system error with custom message" do
+      exception = @s.rb_syserr_new(Errno::ENOENT::Errno, "custom message")
+
+      exception.message.should include("custom message")
+      exception.class.should == Errno::ENOENT
+      exception.should.is_a?(SystemCallError)
+    end
+  end
+
+  describe "rb_syserr_new_str" do
+    it "returns system error with default message when passed message is nil" do
+      exception = @s.rb_syserr_new_str(Errno::ENOENT::Errno, nil)
+
+      exception.message.should include("No such file or directory")
+      exception.class.should == Errno::ENOENT
+      exception.should.is_a?(SystemCallError)
+    end
+
+    it "returns system error with custom message" do
+      exception = @s.rb_syserr_new_str(Errno::ENOENT::Errno, "custom message")
+      exception.message.should include("custom message")
+      exception.class.should == Errno::ENOENT
+      exception.should.is_a?(SystemCallError)
+    end
+  end
+
   describe "rb_make_exception" do
     it "returns a RuntimeError when given a String argument" do
       e = @s.rb_make_exception(["Message"])
