@@ -825,7 +825,7 @@ class TestRefinement < Test::Unit::TestCase
       GC.stress = true
       10.times{
         #{PrependAfterRefine_CODE}
-        undef PrependAfterRefine
+        Object.send(:remove_const, :PrependAfterRefine)
       }
     }, timeout: 60
   end
@@ -1044,7 +1044,7 @@ class TestRefinement < Test::Unit::TestCase
       end
       using Test
       def t
-        'Refinements are broken!'.chop!
+        'Refinements are broken!'.dup.chop!
       end
       t
       module Test
@@ -1824,13 +1824,7 @@ class TestRefinement < Test::Unit::TestCase
       end
     }.refinements
     assert_equal(Integer, refinements[0].target)
-    assert_warn(/Refinement#refined_class is deprecated and will be removed in Ruby 3.4; use Refinement#target instead/) do
-      assert_equal(Integer, refinements[0].refined_class)
-    end
     assert_equal(String, refinements[1].target)
-    assert_warn(/Refinement#refined_class is deprecated and will be removed in Ruby 3.4; use Refinement#target instead/) do
-      assert_equal(String, refinements[1].refined_class)
-    end
   end
 
   def test_warn_setconst_in_refinmenet
@@ -2689,6 +2683,35 @@ class TestRefinement < Test::Unit::TestCase
     assert_equal(:v2, obj.cached_foo_callsite)
   end
 
+  # [Bug #20302]
+  def test_multiple_refinements_for_same_module
+    assert_in_out_err([], <<-INPUT, %w(:f2 :f1), [])
+      module M1
+        refine(Kernel) do
+          def f1 = :f1
+        end
+      end
+
+      module M2
+        refine(Kernel) do
+          def f2 = :f2
+        end
+      end
+
+      class Foo
+        using M1
+        using M2
+
+        def test
+          p f2
+          p f1
+        end
+      end
+
+      Foo.new.test
+    INPUT
+  end
+
   private
 
   def eval_using(mod, s)
